/* Hexwars - A Hexagonal Turn Based Strategy Game
 * Copyright (C) 2009-2011  Patrik Schönfeldt
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include "Client.hpp"

Client::Client(GraphicSet* gs, BasicUserInterface* bui, FSHandler* fsh) {
    LocalGame::init(gs, bui, fsh);
}

Client::Client(GraphicSet* gs, BasicUserInterface* obui,
               Map* omap, UnitContainer* ounits, PLAYER oplyr){
    init(gs, obui, omap, ounits, oplyr);
}

Client::~Client(){
    engine->delChild(cmm);
    delete cmm;
    delete engine;
    delete ui_engine;
}

void Client::init(GraphicSet* gs, BasicUserInterface* obui, Map* omap,
                  UnitContainer* ounits, PLAYER oplyr) {
    map = omap;
    map->setUnitContainer(ounits);
    basicuserinterface = obui;
    units = ounits;
    plyr = oplyr;

    ui_engine = new UIEngine(basicuserinterface->getSize(),Coordinates(0,0),
                             gs,basicuserinterface);
    engine = new Engine(map, units, ui_engine, plyr);

    // add ClientMainMenu
    uim = new UIMenu(Coordinates(128, 256), Coordinates(8,32), gs, ui_engine);
    cmm = new ClientMainMenu( uim, units, plyr);
    engine->addChild(cmm);

    selectedunit = units->selectNextUnit(MAGICIAN, plyr);
    if (selectedunit != NULL) {
        engine->center(selectedunit->getPos());
    }
}

RUNMODE Client::run(){

    switch ( startGameMenu() ) {
    case QUIT:
        return LOBBY;
        break;
    default:
        startGame(levelname);
        break;
    }

    init(graphicset, basicuserinterface, map, units, PLAYER1);

    bool loop = true;
    while (loop){
        currinput = clientLoop();
        switch (currinput) {
        case ENDROUND:
            units->giveRandomOrders(PLAYER2);
            server->endRound();
            server->doMovement();
            break;
        default:
            return LOBBY;
            break;
        }

        /* will need to refresh map and
         * units here if RunMode should be
         * run with a Server */
    }
    delete server;
    return LOBBY;
}

INPUTTYPE Client::clientLoop(){
    cmm->refresh();
    cmm->print();

    /* check if the player has lost
     * by checking if his magician exists */
    if (!units->checkForUnit(MAGICIAN, plyr)){

        /* switch to PLAYER0 so the player cannot play
         * anymore */
        plyr = PLAYER0;

        /* replace menu by a notification that the
         * player has lost the game */
        engine->delChild(cmm);
        delete cmm;
        cmm = new PlayerLooseMenu(uim);
        engine->addChild(cmm);
        cmm->refresh();
        cmm->print();
    }

    map->setPlayerVis(plyr);
    engine->refresh();

    // clientloop
    bool clientloop = true;
    while (clientloop){
        currinput = basicuserinterface->getInput(NOINPUT);
        switch (currinput){
        case NOINPUT:
            engine->print();
            break;
        case QUIT:
            return QUIT;
        default:
            switch(engine->getInput(currinput)){
            case QUIT:
                return QUIT;
            case ENDROUND:
                clientloop = false;
                break;
            case SELECT:
                selectedfield = engine->getSelectedField();
                if ( units->selectUnit(selectedfield) != NULL &&
                     !(units->selectUnit(selectedfield)->isAlive())){
                    units->selectUnit(Coordinates());
                }
                engine->refresh();
                break;
            case COMMAND:
                cmm->print();
                engine->refresh();
                break;
            case SELECTUNIT:
                selectedunit = units->getSelectedUnit();
                cmm->print();
                if (selectedunit != NULL) {
                    engine->center(selectedunit->getPos());
                    engine->refresh();
                }
                break;
            case TIDY:
                cmm->print();
                engine->refresh();
                break;
            case RMB:
                selectedfield = engine->getSelectedField();
                selectedunit = units->getSelectedUnit();
                if (selectedunit != NULL) {
                    if (selectedunit->getOwner() == plyr){
                        selectedunit->command(selectedfield);
                        engine->refresh();
                    }
                } else {
                    selectedunit = units->selectNextUnit(SUMMONER, plyr);
                    selectedunit->command(selectedfield);
                    engine->refresh();
                }
                break;
            case MOVEMENU:
                oldpointerpos = basicuserinterface->getPointerPos();
                while (basicuserinterface->getInput(NOINPUT) == LMBHOLD){
                    pointerpos = basicuserinterface->getPointerPos();
                    if (pointerpos == Coordinates(-1,-1)){
                        pointerpos = oldpointerpos;
                    } else {
                        cmm->move(pointerpos - oldpointerpos);
                        cmm->print();
                    }
                    oldpointerpos = pointerpos;
                }
                break;
            case SCROLL:
                engine->print();
                break;
            default:
                break;
            }
        }
    // end of clientloop
    }
    return ENDROUND;
}

void Client::startGame(std::string level) {
    server = new Server(level, fshandler);
    map = server->getMap(PLAYER0);
    units = server->getUnits(PLAYER0);
}
