/* Hexwars - A Hexagonal Turn Based Strategy Game
 * Copyright (C) 2009-2011  Patrik Schönfeldt
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "Map.hpp"

Map::Map(unsigned int x, unsigned int y, unsigned char h){
    size.x = x;
    size.y = y;
    content = new unsigned short*[x];
    for (unsigned int i=0; i<x; i++){
        content[i] = new unsigned short[y];
    }
    for (unsigned int i=0; i < x; i++) {
        for (unsigned int j=0; j < y; j++) {
            content[i][j] = h;
        }
    }
    waterlevel = h;
}

Map::~Map(){
    for (int i=0; i < size.x; i++){
        delete content[i];
    }
    delete content;
}

Map::Map(const Map& original){
    size = original.size;
    if(size.x <0 || size.y <0){
        throw Hexception("Invalid map size.");
    }
    content = new unsigned short*[size.x];
    for (int i=0; i < size.x; i++){
        content[i] = new unsigned short[size.y];
    }
    waterlevel = original.waterlevel;
    for (int x=0; x < size.x; x++){
    	for (int y=0; y < size.x; y++){
            content[x][y] = original.content[x][y];
    	}
    }
}

void Map::setHeight(unsigned int x, unsigned int y, unsigned char h){
	setHeight(Coordinates(x,y), h);
}

void Map::setHeight(Coordinates pos, unsigned char h){
    if (!(pos < Coordinates(0,0)) && (pos < size)){
        content[pos.x][pos.y] = (content[pos.x][pos.y] & 0xFF00 );
        content[pos.x][pos.y] += h;
    }
}

unsigned char Map::getWaterLevel(){
    return waterlevel;
}

TERRAINTYPE Map::getTerrain(Coordinates pos){
    if ((pos > Coordinates(-1,-1)) && (pos < size)){
        if(waterlevel > content[pos.x][pos.y]%0x100){
    	    return WATER;
        } else {
    	    return GRASS;
        }
    }
    return WATER;
}

void Map::updateSafetyStatus(){
    Coordinates p;
    for(p.x=0; p.x<size.x; (p.x)++){
        for(p.y=0; p.y<size.y; (p.y)++){
            if (getVisStatus(p) < CLEAR){
                setUnsafe(p + Coordinates(0,-1));
                setUnsafe(p + Coordinates(0,+1));
                setUnsafe(p + Coordinates(-1,0));
                setUnsafe(p + Coordinates(+1,0));
                if (p.x %2 == 0){
                    setUnsafe(p + Coordinates(-1,-1));
                    setUnsafe(p + Coordinates(+1,-1));
                } else {
                    setUnsafe(p + Coordinates(-1,+1));
                    setUnsafe(p + Coordinates(+1,+1));
                }
            }
        }
    }
}

void Map::setUnsafe(Coordinates pos){
    if (getVisStatus(pos) == CLEAR){
        setVisStatus(pos, EXPLORED);

        setUnsafe(pos + Coordinates(0,-1));
        setUnsafe(pos + Coordinates(0,+1));
        setUnsafe(pos + Coordinates(-1,0));
        setUnsafe(pos + Coordinates(+1,0));
        if (pos.x %2 == 0){
            setUnsafe(pos + Coordinates(-1,-1));
            setUnsafe(pos + Coordinates(+1,-1));
        } else {
            setUnsafe(pos + Coordinates(-1,+1));
            setUnsafe(pos + Coordinates(+1,+1));
        }
    }
}

void Map::updateVisDex(){
    visdex = 0;
    for (int i = 0; i < size.x; i++) {
        for (int j = 0; j < size.y; j++) {
            visdex += getVisStatus(i,j);
        }
    }
}

void Map::setVisStatus(Coordinates pos, VISIBILITY stat){
    if (!(pos < Coordinates(0,0)) && pos < size){
        content[pos.x][pos.y] = (content[pos.x][pos.y] & 0xFCFF);
        content[pos.x][pos.y] += stat * 0x100;
    }
}

void Map::updateVisStatus(Coordinates pos, VISIBILITY stat){
    if (stat > getVisStatus(pos)) {
        if (!(pos << Coordinates(0,0)) && pos < size){
            content[pos.x][pos.y] = (content[pos.x][pos.y] & 0xFCFF);
            content[pos.x][pos.y] += stat * 0x100;
        }
    }
}

VISIBILITY Map::getVisStatus(Coordinates pos){
    if (!(pos << Coordinates(0,0)) && pos < size){
        return ((content[pos.x][pos.y] & 0x0300)/0x100);
    } else {
        return UNKNOWN;
    }
}


VISIBILITY Map::getVisStatus(int x, int y){
    return getVisStatus(Coordinates(x,y));
}

void Map::sync(Map* orig){
    for (int i=0; i < size.x; i++) {
        for (int j=0; j < size.y; j++) {
            content[i][j] = (orig->content[i][j] & 0x00FF)
                          + (content[i][j] & 0xFF00);
        }
    }
}

std::vector<Coordinates> Map::way(Coordinates from,Coordinates to) {
    std::vector<Coordinates> way;

    switch (from.distanceTo(to)) {
    case 0:
        break;
    case 1:
        way.push_back(to);
        break;
    default:
        way = aStarPath(from, to);
        break;
    }

    return way;
}

std::string Map::getAscii(){
    std::stringstream out;
    for (int i=0; i < size.x; i++) {
        for (int j=0; j < size.y; j++) {
            if (i%2 == 1) {
                out << ' ';
            }
            out << (int) getHeight(i,j);
            if (i%2 == 0) {
                out << ' ';
            }
        }
        out << '\n';
    }
    return out.str();
}

std::vector<Coordinates> Map::aStarPath(Coordinates from, Coordinates to){
    // open list
    std::list<PathNode*> ol;
    // closed list
    std::list<PathNode*> cl;
    PathNode* currentnode;

    PathNode* newnode;
    Coordinates tmpfield;
    Coordinates tmpneighbour;

    // in list? (used for open and closed list)
    bool inl;
    // list iterator (used for open and closed list)
    std::list<PathNode*>::iterator lit;

    std::vector<Coordinates> path;

    currentnode = new PathNode(from, NULL, to);
    ol.push_back( currentnode );

    while (! ol.empty() ) {
        currentnode = ol.front();
        cl.push_back(currentnode);
        ol.pop_front();
#ifdef DEBUG
        currentnode->check();
#endif

        tmpfield = currentnode->getField();

        if (tmpfield == to) {
            // delete entries from open list
            for (lit = ol.begin(); lit != ol.end(); lit++) {
                delete *lit;
            }
            // reconstruct path
            while (currentnode->getField() != from) {
                path.push_back(currentnode->getField());
                currentnode = currentnode->getLast();
            }
            // delete entries from closed list
            for (lit = cl.begin(); lit != cl.end(); lit++) {
                delete *lit;
            }
            std::reverse(path.begin(),path.end());
            return path;
        }

        for (int x=-1; x<=1; x++) {
            for (int y=-1; y<=1; y++) {
                tmpneighbour = tmpfield + Coordinates(x,y);
                if ( getTerrain(tmpneighbour) != WATER
                        && tmpfield.distanceTo(tmpneighbour) == 1) {
                    // check weather tmpneigbour has already been analyzed
                    inl = false;
                    for (lit = cl.begin(); lit != cl.end(); lit++) {
                        if ((*lit)->getField() == tmpneighbour) {
                            inl = true;
                            break;
                        }
                    }
                    if (!inl) {
                        newnode = new PathNode(tmpneighbour, currentnode, to);
#ifdef DEBUG
                        newnode->check();
#endif
                        for (lit = ol.begin(); lit!= ol.end(); lit++) {
                            if ((*lit)->getField() == newnode->getField()) {
                                inl = true;
                                break;
                            }
                        }
                        if (inl) {
                            if (newnode->getCost() < (*lit)->getCost()) {
                                (*lit)->setLast(currentnode);
                            }
                            delete newnode;
                        } else {
                            lit = ol.begin();
                            while( lit != ol.end() &&
                                  ((*lit)->getCost() < newnode->getCost()) ) {
                                lit++;
                            }
                            ol.insert(lit,newnode);
                        }
                    }
                }
            }
        }
    }
    // no path found
    return path;
}

void Map::setPlayerVis(PLAYER p){
    Coordinates c1, c2;

    if(p == PLAYER0){
        for(c1.x=0; c1.x<size.x; (c1.x)++){
            for(c1.y=0; c1.y<size.y; (c1.y)++){
                setVisStatus(c1, VISIBLE);
            }
        }
    } else {
        /* set all VISIBLE fields CLEAR; of course fields neighboring
         * unsafe fields are not save, so updateSafetyStatus() is called below
         */
        for(c1.x=0; c1.x<size.x; (c1.x)++){
            for(c1.y=0; c1.y<size.y; (c1.y)++){
                if (getVisStatus(c1) == VISIBLE){
                    setVisStatus(c1, CLEAR);
                }
            }
        }

        /* check if fields are visible
         * and mark them */
        for(unsigned int u=1; u <= units->getLatestUuid(); u++){
            Unit *unit = units->getUnit(u);
            if (unit != NULL){
                if (unit->getOwner() == p && unit->getOldHp() > 0){
                    c2 = unit->getPos();
                    for(c1.x=0; c1.x<size.x; (c1.x)++){
                        for(c1.y=0; c1.y<size.y; (c1.y)++){
                            if( unit->getSight() >= c2.distanceTo(c1)) {
                                if ( unit->getSight() > c2.distanceTo(c1)) {
                                    updateVisStatus(c1, VISIBLE);
                                } else {
                                    if (getVisStatus(c1) == UNKNOWN){
                                        updateVisStatus(c1, EXPLORED);
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }

        updateSafetyStatus();
    }
}
