/* Hexwars - A Hexagonal Turn Based Strategy Game
 * Copyright (C) 2009-2011  Patrik Schönfeldt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "MapLoader.hpp"

VirtualMapLoader::VirtualMapLoader() {
    map = NULL;
    player = NULL;
}

void VirtualMapLoader::loadMap(std::string levelname) {
    // delete current data before loading a new map
    delete map;
    delete player;

    PathName path = levelname;
    if (!fsh->exists(path)) {
        path = "maps/";
        path /= levelname;
    }

    try{
        path = fsh->getFullPath(path);
    }
    catch(Hexception& e){
        std::cerr << e.what();
        exit(1);
    }
    loadMapFromDisk(path);
}

void VirtualMapLoader::saveMap(std::string savegame) {
    prepareSavePath();

    PathName path = savepath;
    path /= savegame;
    saveMapToDisk(path, 2);
}

Map* VirtualMapLoader::getMap(){
    return map;
}

void VirtualMapLoader::setMap(Map* omap) {
    map = omap;
}

UnitContainer* VirtualMapLoader::getUnits(){
    return player;
}

void VirtualMapLoader::setUnits(UnitContainer* ounits) {
    player = ounits;
}


/* HWmap_V01 format saves owner of a unit in
 * the field before the last digit of a decimal
 * number. */
int VirtualMapLoader::hw01_owner(int feld){
    return ((feld%100 - feld%10)/10);
}

/* HWmap_V01 format saves unit (and terrain!)
 * in the last digit of a decimal number. */
int VirtualMapLoader::hw01_unit(int feld){
    return (feld%10);
}

void VirtualMapLoader::decode_HWmap_V01(unsigned int tmpmap[32][32]) {
    unsigned int X = map->getSize().x;
    unsigned int Y = map->getSize().y;
    for (unsigned int x=0; x<X; x++){
        for (unsigned int y=0; y<Y; y++){
            /* HWmap_V01 only can handle Units of PLAYER1
             * and PLAYER2. PLAYER0 is reserved for Terrain. */
            if (hw01_owner(tmpmap[x][y]) != PLAYER0){
                switch (hw01_unit(tmpmap[x][y])){
                case SPIRIT:
                    player->saveUnit(new Spirit(      Coordinates(x,y),
                                                      hw01_owner(tmpmap[x][y])));
                    break;
                case MAGICIAN:
                    player->saveUnit(new Magician(    Coordinates(x,y),
                                                      hw01_owner(tmpmap[x][y])));
                    player->saveUnit(new Summoner(    Coordinates(x,y),
                                                      hw01_owner(tmpmap[x][y])));
                    break;
                case SCOUT:
                    player->saveUnit(new Scout(       Coordinates(x,y),
                                                      hw01_owner(tmpmap[x][y])));
                    break;
                case TERRAFORMER:
                    player->saveUnit(new TerraFormer( Coordinates(x,y),
                                                      hw01_owner(tmpmap[x][y])));
                    break;
                case WARRIOR:
                    player->saveUnit(new Warrior(     Coordinates(x,y),
                                                      hw01_owner(tmpmap[x][y])));
                    break;
                default:
                    break;
                }
            }
            /* HWmap_V01 saves water as 0, everything else
             * must be assumed as height 1 since units are
             * saved in the map. */
            if (tmpmap[x][y] == 0) {
                map->setHeight(x,y,0);
            } else {
                map->setHeight(x,y,1);
            }
        }
    }
}

void VirtualMapLoader::decode_HWmap_V02(std::string chars) {
    std::stringstream mapstring(chars.c_str());
    unsigned int X = map->getSize().x;
    unsigned int Y = map->getSize().y;
    unsigned int h;
    for (unsigned int x=0; x<X; x++){
        for (unsigned int y=0; y<Y; y++){
            mapstring >> h;
            map->setHeight(x,y,h);
        }
    }
    Unit* u = NULL;
    unsigned int uuid;
    PLAYER owner;
    UNITTYPE type;
    Coordinates pos;
    short hp, oldhp;
    char junk[256];
    while (mapstring.good()){
        // read values of the unit
        mapstring >> uuid;
        mapstring >> junk;
        mapstring >> owner;
        mapstring >> junk;
        mapstring >> type;
        mapstring >> junk;
        mapstring >> pos.x;
        mapstring >> junk;
        mapstring >> pos.y;
        mapstring >> junk;
        mapstring >> hp;
        mapstring >> junk;
        mapstring >> oldhp;
        mapstring.getline(junk, sizeof(junk));

        // create unit
        switch (type){
        case SPIRIT:
            u = new Spirit( pos, owner );
            break;
        case MAGICIAN:
            u = new Magician( pos, owner );
            break;
        case SCOUT:
            u = new Scout( pos, owner );
            break;
        case TERRAFORMER:
            u = new TerraFormer ( pos, owner );
            break;
        case WARRIOR:
            u = new Warrior( pos, owner );
            break;
        default:
            u = NULL;
            break;
        }
        if (u != NULL) {
            u->setUuid( uuid );
            u->changeHp( hp - u->getHp() );
            player->saveUnit(u);
        }
    }
}

std::string VirtualMapLoader::encode_HWmap_V01() {
    std::stringstream mapstring;
    mapstring << "HWmap_V01 " << map->getSize().x
              << " "          << map->getSize().y << std::endl;
    for (int y=0; y < map->getSize().y; y++) {
        for (int x=0; x < map->getSize().x; x++) {
            unit = player->getUnit(Coordinates(x,y));
            if (unit == NULL) {
                switch(map->getTerrain(Coordinates(x,y))) {
                case WATER:
                    mapstring << "00";
                    break;
                case GRASS:
                    mapstring << "01";
                    break;
                default:
                    mapstring << "00";
                    break;
                }
            } else {
                mapstring << unit->getOwner();
                mapstring << unit->getType();
            }
        if (x < map->getSize().x - 1)
            mapstring << " ";
        }
        mapstring << std::endl;
    }
    return mapstring.str();
}

std::string VirtualMapLoader::encode_HWmap_V02() {
    std::stringstream mapstring;
    mapstring << "HWmap_V02 " << map->getSize().x
              << " "          << map->getSize().y << std::endl;
    mapstring << std::endl << map->getAscii();
    mapstring << std::endl << player->getAscii();
    mapstring << std::endl;

    return mapstring.str();
}

void VirtualMapLoader::prepareSavePath() {
    savepath = fsh->getSettingsDir();
    fsh->prepareDirectory(savepath);

    savepath /= "save/";
    fsh->prepareDirectory(savepath);
}
