/* Hexwars - A Hexagonal Turn Based Strategy Game
 * Copyright (C) 2009-2011  Patrik Schönfeldt
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#ifndef __IE_HPP__
#define __IE_HPP__

#include <vector>
#include <sstream>

#include "UI.hpp"

/**
 * \brief IE, a interaction entity, is an entity that is
 * responsable for logical user interaction.
 *
 * An IE is the thing that is what you actually will believe
 * to see on screen. Technically the IE will trigger a UI to
 * represent the IE but the IE will do the logical things,
 * calculations, will determine what to show where, etc.
 * IE is written independend from whatever UI lib you use.
 * UI will do the lib-dependent tasks.<br>
 * IE is a pure virtual class.
 */

class IE{
public:
    /** \brief does nothing
     *
     * Does nothing. Just defined for inheritance.
     */
    IE();

    /** \brief give UI to display the IE
     *
     * Allows you to set the UI used to display
     * this logical element and which is capabale
     * of giving user input to it.
     */
    IE(UI* oui);

    /** \brief deletes childs
     *
     */
    ~IE();

    /** \brief Give childs to the IE.
      */
    void addChild(IE* cld);

    /** \brief removes IE from the list of childs
     *
     * delChild(IE*) does not delete the IE itself
     */
    void delChild(IE* cld);

    /** \brief adds an IE* as (fullscreen) massage
     */
    void addMsg(IE* msg);

    /** \brief shows the n'th IE added
     *
     * shows usual IE again when you give n<0
     */
    void printMsg(int n);

    /** \brief just refresh, do not print
     *
     * refreshes all the contents seen on the parents
     * output. This will NOT draw anything
     * on the screen.
     */
    virtual void refresh() = 0;

    /** \brief lets the attached UI print
     *
     * just triggers the UI to print the output
     */
    void print();

    /** \brief move the IE by dp
     *
     */
    void move(Coordinates dp);

    /** \brief processes given input
     *
     * first asks childs to process given input,
     * then tries for specificInput()
     */
    INPUTTYPE getInput(INPUTTYPE parinput);

    std::string convertInt(int number);

protected:
    /** \brief processes given input
     *
     * Every implementation of IE will need a possibility
     * for specific input. Since standard getInput() asks
     * all childs for input this is done by this extra
     * method to be sure that nobody overwrites non specific
     * parts of getInput().
     */
    virtual void specificInput(INPUTTYPE) =0;

    /** \brief cache for current input
     *
     * this is the unprocessed information
     */
    INPUTTYPE currinput;

    /** \brief saves last input
     *
     * can be used to compare INPUTTYPEs
     */
    INPUTTYPE lastinput;

    /**
     * every IE needs it UI to be displayed and to
     * receive user input
     */
    UI* myui;

    /** \brief saves childs
     *
     * child IEs are displayed on top of the parent IE
     * (p.e. menu in a window)
     */
    std::vector<IE*> child;


    std::vector<IE*> msg;
    IE* child0;
};



#endif /* __IE_HPP__ */
