/* Hexwars - A Hexagonal Turn Based Strategy Game
 * Copyright (C) 2009-2011  Patrik Schönfeldt
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include "Al4BasicUserInterface.hpp"

Al4BasicUserInterface::Al4BasicUserInterface(FSHandler* ofsh){
    fsh = ofsh;
    pos = Coordinates(0,0);

    prepareUICfgPath();

    try{
        loadUIConfig();
    }
    catch(Hexception& e){
        // set default values
        SFull = false;
        size.x = 640;
        size.y = 480;
        SDepth = 16;
        if (!uicfgpath.exists()){
            saveUIConfig();
            std::cout << "Warning: No configuration file found. Created default at '"
                      << uicfgpath.string() << "'.\n";
        } else {
            std::cerr << e.what();
        }
    }
    // initalize allegro
    allegro_init();
    // set screen resolution
    setGraphicMode();
    // make keyboard usable
    install_keyboard();
    // make mouse usable
    install_mouse();
    set_mouse_range(0, 0, SCREEN_W, SCREEN_H);
    init(size, pos, NULL, NULL);


    char exename[128];
    get_executable_name(exename,sizeof(exename));
    fsh->setExeDir(PathName(exename));
}

Al4BasicUserInterface::~Al4BasicUserInterface(){
}

void Al4BasicUserInterface::setGraphicMode(){
    set_color_depth(SDepth);
    if(SFull){
        set_gfx_mode(GFX_AUTODETECT_FULLSCREEN, size.x, size.y, 0, 0);
    }
    else{
        set_gfx_mode(GFX_AUTODETECT_WINDOWED, size.x, size.y, 0, 0);
    }
    return;
}

INPUTTYPE Al4BasicUserInterface::getInput(INPUTTYPE currinput){
    usleep(20000);
    if (key[KEY_ESC] || key[KEY_Q]){
        if (lastinput != QUIT){
            lastinput = QUIT;
            return QUIT;
        }
        return NOINPUT;
    }
    if (mouse_b & 0x01){
        if (lastinput != LMB && lastinput !=LMBHOLD){
            lastinput = LMB;
            return LMB;
        }
        lastinput = LMBHOLD;
        return LMBHOLD;
    }
    if (mouse_b & 0x02){
        if (lastinput != RMB && lastinput !=RMBHOLD){
            lastinput = RMB;
            return RMB;
        }
        lastinput = RMBHOLD;
        return RMBHOLD;
    }
    if (key[KEY_W] || key[KEY_UP]){
        if (lastinput != UP){
            lastinput = UP;
            return UP;
        }
        return NOINPUT;
    }
    if (key[KEY_S] || key[KEY_DOWN]){
        if (lastinput != DOWN){
            lastinput = DOWN;
            return DOWN;
        }
        return NOINPUT;
    }
    if (key[KEY_A] || key[KEY_LEFT]){
        if (lastinput != LEFT){
            lastinput = LEFT;
            return LEFT;
        }
        return NOINPUT;
    }
    if (key[KEY_D] || key[KEY_RIGHT]){
        if (lastinput != RIGHT){
            lastinput = RIGHT;
            return RIGHT;
        }
        return NOINPUT;
    }
    if (key[KEY_ENTER] || key[KEY_SPACE]){
        if(lastinput != SELECT){
            lastinput = SELECT;
            return SELECT;
        }
        return NOINPUT;
    }
    if (key[KEY_1_PAD]){
        if(lastinput != KP1){
            lastinput = KP1;
            return KP1;
        }
        return NOINPUT;
    }
    if (key[KEY_2_PAD]){
        if(lastinput != KP2){
            lastinput = KP2;
            return KP2;
        }
        return NOINPUT;
    }
    if (key[KEY_3_PAD]){
        if(lastinput != KP3){
            lastinput = KP3;
            return KP3;
        }
        return NOINPUT;
    }
    if (key[KEY_4_PAD]){
        if(lastinput != KP4){
            lastinput = KP4;
            return KP4;
        }
        return NOINPUT;
    }
    if (key[KEY_5_PAD]){
        if(lastinput != KP5){
            lastinput = KP5;
            return KP5;
        }
        return NOINPUT;
    }
    if (key[KEY_6_PAD]){
        if(lastinput != KP6){
            lastinput = KP6;
            return KP6;
        }
        return NOINPUT;
    }
    if (key[KEY_7_PAD]){
        if(lastinput != KP7){
            lastinput = KP7;
            return KP7;
        }
        return NOINPUT;
    }
    if (key[KEY_8_PAD]){
        if(lastinput != KP8){
            lastinput = KP8;
            return KP8;
        }
        return NOINPUT;
    }
    if (key[KEY_9_PAD]){
        if(lastinput != KP9){
            lastinput = KP9;
            return KP9;
        }
        return NOINPUT;
    }
    lastinput = NOINPUT;
    return NOINPUT;
}

void Al4BasicUserInterface::loadUIConfig(){
    std::ifstream uisettings;
    uisettings.open(uicfgpath.c_str());
    if(!uisettings.good()){
        throw Hexception( std::string("Error loading config file: '")
                          + uicfgpath.string()
                          + std::string("'. Will use default values.\n")
                        );
        return;
    }
    char tmpchar;
    std::string descripiton;
    while(uisettings.good()){
        descripiton.clear();
        for(int i=0; i<11 ; i++){
            uisettings >> tmpchar;
            descripiton.push_back(tmpchar);
        }
        if(descripiton == "sfullscreen")
            uisettings >> SFull;
        if(descripiton == "screenwidth")
            uisettings >> size.x;
        if(descripiton == "screenhight")
            uisettings >> size.y;
        if(descripiton == "scolordepth")
            uisettings >> SDepth;
    }
        if(size.x <= 0 || size.y <= 0){
        throw Hexception( "Error: Invalid configuration. "
                          "Will use default values.\n");
        return;
    }
}

void Al4BasicUserInterface::saveUIConfig(){
    std::ofstream uisettings;
    uisettings.open(uicfgpath.c_str());
    uisettings << "screenwidth " << size.x << std::endl;
    uisettings << "screenhight " << size.y << std::endl;
    uisettings << "sfullscreen " << SFull << std::endl;
    uisettings << "scolordepth " << SDepth << std::endl;
    uisettings.close();
}

void Al4BasicUserInterface::setFullscreen(bool full){
    SFull = full;
}

void Al4BasicUserInterface::setResolution(int width, int hight, int depth){
    setResolution(Coordinates(width, hight), depth);
}

void Al4BasicUserInterface::setResolution(Coordinates size, int depth){
    setSize(size);
    SDepth = depth;
}

bool Al4BasicUserInterface::getFullscreen(){
    return SFull;
}

void Al4BasicUserInterface::drawPointer(){
    masked_blit( graphics->cursor[0], buffer, 0, 0, mouse_x, mouse_y,
                 (graphics->cursor[0])->w, (graphics->cursor[0])->h);
}

void Al4BasicUserInterface::print(){
    clear_bitmap(buffer);
    masked_blit( TLB, buffer, 0, 0, 0, 0, size.x, size.y);
    drawPointer();
    blit( buffer, screen, 0, 0, 0, 0, size.x, size.y);
}

void Al4BasicUserInterface::prepareUICfgPath(){
    uicfgpath = fsh->getSettingsDir();

    fsh->prepareDirectory(uicfgpath);

    uicfgpath /= "/cfg";
    fsh->prepareDirectory(uicfgpath);

    uicfgpath /= "/uisettings.cfg";
}
