/* Hexwars - A Hexagonal Turn Based Strategy Game
 * Copyright (C) 2009-2011  Patrik Schönfeldt
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#ifndef __AL4UI__HPP__
#define __AL4UI__HPP__

#include <allegro.h>
#include <vector>

#include "../../definitions/Coordinates.hpp"
#include "../MenuItem.hpp"
#include "../GraphicSet.hpp"
#include "../virtual/VirtualUI.hpp"


/** \brief UIs are responsable for representing interaction
 * entities (like menus) to the user. UIs can be seen as an
 * abstraction layer for several user interface solutions
 * (like allegro, X11, etc.)
 */
class Al4UI : public VirtualUI {
public:
    Al4UI();

    /** \brief Constructor needs parent UI, input will be set to NULL
     *
     * size defines the size of the
     * UI on screen, the constructor will create
     * a BITMAP for buffering in the given
     * size
     *
     * pos defines the position of the
     * UI on the parents input
     *
     * UI needs a GraphicSet* to be able to print
     * graphics, link one with gs
     *
     * UI* par defines a parent UI, that this
     * UI will be printed on
     *
     * WARNING: input is set to NULL by constructor
     * because it is only needed if there are child
     * UIs
     */
    Al4UI(Coordinates size, Coordinates pos, GraphicSet* gs, Al4UI* par);

    /** \brief destroys bitmaps
     */
    ~Al4UI();

    void setGraphicSet(GraphicSet* gs);

    /** \brief Locks or unlocks the double buffer.
     *
     * With some graphic libs a higher drawing performance may be
     * accomplished when drawing on a locked bitmap. Locked bitmaps
     * in allegro can not be read but only written.
     */
    void lockScreen(bool doit);

    /** \brief Returns the position of the pointer.
     *
     * Returns where the mouse points at in the current UI.
     * Will return Coordinates(-1,-1) when mouse does not point 
     * in into the area of this UI. Uses allegro
     * (mouse_x, mouse_y) position and size of the UI. 
     *
     * \returns Coordinates of the mouse on the current UI
     */
    Coordinates getPointerPos();

    /** \brief Prints buffer and input on parents input.
     *
     * First of all it prints scrollbuffer, the
     * staticbuffer (the one with transparency)
     * and the pointer on the screenbuffer. Then
     * the screenbuffer is printed on screen.
     */
    virtual void print();

    void clearBlack();

    /** \brief Clears the screen.
     *
     * Clears the buffer to magic pink.
     */
    void clear();

    /** \brief clears top layer buffer
     *
     */
    void clearTLB();

    /** \brief This method analyzes if user input is importent
     * for current UI.
     *
     * It will return NOINPUT if nothing important happened.
     * This basic implementation just returns the return HOVER
     * if pointer is above UI.
     *
     * \returns code for user input: HOVER or NOINPUT
     */
    virtual INPUTTYPE getInput(INPUTTYPE currinput);

    /** \brief moves UI around on parent UI
     *
     */
    void changePos(Coordinates dp);

    /** \brief Returns the size of the window.
     *
     * Returns the size of the window that is
     * currently displayed.
     *
     * \returns size of the UI
     */
    Coordinates getSize();

    /** \brief change size on screen
     *
     */
    void setSize(Coordinates s);

    /** \brief Children will draw their buffers onto this BITMAP.
     *
     * This is the top layer buffer. If there are child UIs they
     * will use this buffer to draw on parent UIs. After printing
     * its own doublebuffer on screen (or onto the parent TLB) this
     * input will be printed. This means that only contents that
     * really need a refresh will be refreshed.
     */
    BITMAP* TLB;

    void printText(std::string text, Coordinates pos);

protected:
    /** \brief function for initialization
     * it is a good idea to use this in your constructor of child classes
     */
    void init(Coordinates size, Coordinates pos, GraphicSet* gs, Al4UI* par);

    /** \brief saves last input
     */
    INPUTTYPE lastinput;

    /** \brief saves the size of the UI on parents input
     */
    Coordinates size;

    /** \brief saves the position of the UI on parents input
     */
    Coordinates pos;

    /** \brief internal buffer — only redraw contents if refresh is needed
     */
    BITMAP* buffer;

    /** \brief storage container for single elements to draw on screen
     */
    GraphicSet* graphics;

    /** \brief every UI has a parent UI (parent window)
     *
     * Every UI is nested in anoter UI. The basis of this is a special
     * implementation of UI, the BasicUserInterface. It has no parent
     * but will use pure allegro routines.
     */
    Al4UI* parent;
};

#endif /* __AL4UI__HPP__ */
