/* Hexwars - A Hexagonal Turn Based Strategy Game
 * Copyright (C) 2009-2012  Patrik Schönfeldt
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#ifndef __Al5BasicUserInterface_HPP__
#define __Al5BasicUserInterface_HPP__

class Al5BasicUserInterface;
typedef Al5BasicUserInterface BasicUserInterface;

#include <string>
#include <ostream>
#include <stdlib.h>
#include <fstream>
#ifdef DEBUG
#include <sstream>
#endif

#include <allegro5/allegro.h>
#include <allegro5/allegro_native_dialog.h>
#include <allegro5/allegro_font.h>
#include <allegro5/allegro_ttf.h>
#include <allegro5/allegro_primitives.h>
#include <allegro5/allegro_image.h>

#include "../UI.hpp"
#include "../virtual/VirtualBasicUserInterface.hpp"
#include "../../misc/Exceptions.hpp"
#include "../../definitions/Coordinates.hpp"
#include "../../filesystem/PathName.hpp"
#include "../../filesystem/FSHandler.hpp"

/** \brief responsable for display and input operations
 *
 *  Al5BasicUserInterface is responsable for recognizing basic
 *  user inputs (defined in UI) and for managing Hexwars
 *  display or window. It provides the ground UI::buffer
 *  which is used as a double buffer.
 *
 */


class Al5BasicUserInterface : public UI, public VirtualBasicUserInterface {
public:
    Al5BasicUserInterface(FSHandler* ofsh);

    ~Al5BasicUserInterface();

    void setFullscreen(bool);
    bool getFullscreen();

    inline void setResolution(int, int, int);
    void setResolution(Coordinates, int);


    /** \brief does nothing until input (or timer)
     *
     *  Since we have a timer running, there will be at least
     *  $(FPS) events every second that cause a redraw. If the
     *  player is highly active, there might be more to do ^^.
     *  Returns user input done by keys.
     *
     *  This will return a keycode as defined in UI if the player
     *  presses one of the keys bind to this action. If no key
     *  is pressed for a time defined by FPS it will return TIME
     *  instead.
     *
     * \returns code for user input
     */
    INPUTTYPE getInput(INPUTTYPE currinput);

    /**
     * Sets the screen resolution, fullscreen, etc.
     * You need to call this after switching resolution
     * et al.
     */
    void setGraphicMode();

    void setGraphicSet(GraphicSet* gs);

    void print();

    void saveUIConfig();
    
    void message(const char* msg){
       al_show_native_message_box(NULL, NULL, NULL, msg, NULL, 0);
    }

private:
    /** Display(s)/ screen(s) has to be initialized seperatly in
     *  Allegro 5.
     */
    ALLEGRO_DISPLAY *screen;

    ALLEGRO_EVENT_QUEUE *event_queue;
    ALLEGRO_TIMER *timer;

    /** fullscreen */
    bool SFull;

    /** color depth */
    int SDepth;

    /** targeted frames per second */
    float FPS;

    /** loads config file for UI config data,
     *  it includes default resolution et al.
     */
    void loadUIConfig();

    FSHandler* fsh;

    PathName uicfgpath;

    INPUTTYPE checkMouseHeld();

    void prepareUICfgPath();

#ifdef DEBUG
    double lasttime;
    double currtime;
    unsigned int fps;
    std::string fpstext;
    unsigned int framesdone;
#endif
};

#endif /* __Al5BasicUserInterface_HPP__ */
