/* Hexwars - A Hexagonal Turn Based Strategy Game
 * Copyright (C) 2009-2012  Patrik Schönfeldt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef VIRTUALUI_HPP
#define VIRTUALUI_HPP

typedef unsigned int INPUTTYPE;
/**
 * NOINPUT: nothing happened
 * LMB: left mouse button klicked
 * RMB: right mouse button clicked
 * MMB: middle mouse button clicked
 * LMBHOLD: left mouse button not released
 * RMBHOLD: right mouse button not released
 * UP: uparrow hit
 * DOWN: downarrow hit
 * SCROLL: scrolled the map or changed item in a menu
 * SELECT: selected something (to be run instantly)
 * QAl5UIT: qAl5UIt the game or wharever
 * ENDROUND: end a round
 * MOVEMENU: changed the position of a movable menu
 * SWITCHPLAYER: initialize switching players
 * SWITCH2P1: active player is switched to PLAYER1
 * SWITCH2P1: active player is switched to PLAYER2
 */
enum Inputtypes{
    NOINPUT=0, QUIT=1, TIME=3, LMB, LMBHOLD, RMB, RMBHOLD, MMB, MMBHOLD,
    ENDROUND, MOVEMENU, UP, DOWN, LEFT, RIGHT, HOVER, SAVE, SWITCHPLAYER,
    SWITCH2P1, SWITCH2P2, KP1, KP2, KP3, KP4, KP5, KP6, KP7, KP8, KP9,
    MOVE, COMMAND, START, SELECTUNIT, TIDY, MENUSELECT, SCROLL, SELECT,
    SAVEGAME, CONTINUEGAME, P0AICMD, P1AICMD, P2AICMD
};

#include <string>

#include "../GraphicSet.hpp"
#include "../../definitions/Coordinates.hpp"

/** \brief UIs are responsable for representing interaction
 * entities (like menus) to the user. Al5UIs can be seen as an
 * abstraction layer for several user interface solutions
 * (like allegro, X11, etc.)
 */
class VirtualUI {
public:
    virtual void setGraphicSet(GraphicSet* gs) =0;

    /** \brief Just needed for campatibily reasons */
    virtual void lockScreen(bool doit) =0;

    /** Allegro-5-version of Hexwars does not support mouse yet!
     *
     * \returns Coordinates(-1,-1)
     */
    virtual Coordinates getPointerPos() =0;

    /** \brief Prints buffer and input on parents input.
     *
     * First of all it prints scrollbuffer, the
     * staticbuffer (the one with transparency)
     * and the pointer on the screenbuffer. Then
     * the screenbuffer is printed on screen.
     */
    virtual void print() =0;

    virtual void clearBlack() =0;

    /** \brief Clears the screen.
     *
     * Clears the buffer to magic pink.
     */
    virtual void clear() =0;

    /** \brief clears top layer buffer
     *
     */
    virtual void clearTLB() =0;

    /** \brief This method analyzes if user input is importent
     * for current UI.
     *
     * It will return NOINPUT if nothing important happened.
     * This basic implementation just returns the return HOVER
     * if pointer is above UI.
     *
     * \returns code for user input: HOVER or NOINPUT
     */
    virtual INPUTTYPE getInput(INPUTTYPE currinput) =0;

    /** \brief moves UI around on parent Al5UI
     *
     */
    virtual void changePos(Coordinates dp) =0;

    /** \brief Returns the size of the window.
     *
     * Returns the size of the window that is
     * currently displayed.
     *
     * \returns size of the UI
     */
    virtual Coordinates getSize() =0;

    /** \brief change size on screen
     *
     */
    virtual void setSize(Coordinates s) =0;

    virtual void printText(std::string text, Coordinates pos) =0;
};

#endif // VIRTUALUI_HPP
