/* Hexwars - A Hexagonal Turn Based Strategy Game
 * Copyright (C) 2009-2011  Patrik Schönfeldt
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include "UnitContainer.hpp"

UnitContainer::UnitContainer(Map* omap) {
    map = omap;
    latestuuid = 0;
    selectedunit = -1;
    unit.clear();
}

UnitContainer::UnitContainer(const UnitContainer& orig) {
    latestuuid = orig.latestuuid;
    map = orig.map;
    selectedunit = -1;
    unit.clear();
    for( unsigned int i=0; i<orig.unit.size(); i++) {
        unit.push_back(new Unit(*(orig.unit[i])));
    }
}

UnitContainer::~UnitContainer() {
    for (unsigned int i=0; i<unit.size(); i++){
        delete unit[i];
    }
    unit.clear();
}

#ifdef DEBUG
void UnitContainer::check() {
    for (unsigned int i=0; i<unit.size(); i++){
        unit[i]->check();
    }
}
#endif

unsigned int UnitContainer::getLatestUuid() {
    return latestuuid;
}

Unit* UnitContainer::getUnit(unsigned int uuid) {
    for(unsigned int i = 0; i<unit.size(); i++){
        if( (unit[i])->getUuid() == uuid){
            return (unit[i]);
        }
    }
    return NULL;
}

void UnitContainer::newRoundPoke() {
    for(unsigned int i = 0; i<unit.size(); i++){
        unit[i]->newRoundPoke();
    }
}

void UnitContainer::doMovement() {
    newRoundPoke();
    unsigned int time;
    for(int i=0; i<5; i++){
        switch(i){
        case 0:
            time = 25;
            break;
        case 1:
            time = 33;
            break;
        case 2:
            time = 50;
            break;
        case 3:
            time = 66;
            break;
        case 4:
            time = 75;
            break;
        default:
            break;
        }
        for(unsigned int i = 0; i<unit.size(); i++){
            if (unit[i]->isAlive()) {
                unit[i]->move(time);
            }
        }
        for(unsigned int i = 0; i<unit.size(); i++){
            for(unsigned int j = 0; j<i; j++){
                collide(unit[i], unit[j]);
            }
        }
    }
    merge();
}

void UnitContainer::merge() {
    for(unsigned int i = 0; i<unit.size(); i++){
        for(unsigned int j = 0; j<i; j++){
            Unit* u1 = unit[i];
            Unit* u2 = unit[j];
            if((u1->getPos()  ==  u2->getPos()) &&
                u1->isAlive() &&  u2->isAlive() )
            {
                if( u1->getHp() > u2->getHp()){
                    u1->changeHp(u2->getHp());
                    u2->kill();
                } else {
                    u2->changeHp(u1->getHp());
                    u1->kill();
                }
            }
        }
    }
}

void UnitContainer::tidy() {
    for(unsigned int i = 0; i<unit.size(); i++){
        unit[i]->delOldWay();
        if( !unit[i]->isAlive() && unit[i]->getType() != SUMMONER ){
            delete unit[i];
            unit.erase(unit.begin()+i);
            i--;
        }
    }
    if (selectedunit >= (int) unit.size()) {
        selectedunit = -1;
    }
}

void UnitContainer::clearCommands() {
    for(unsigned int i = 0; i<unit.size(); i++){
        unit[i]->clearCommands();
    }
}

void UnitContainer::sync(UnitContainer* orig) {
    if (latestuuid < orig->latestuuid) {
        latestuuid = orig->latestuuid;
    } else {
        orig->latestuuid = latestuuid;
    }
    // iteration thru orig's units
    for( unsigned int i=0; i<orig->unit.size(); i++){
        // load unit with same uuid
        Unit* tmp = getUnit(orig->unit[i]->getUuid());
        if (tmp != NULL){
            // sync it, if it is already known
            tmp->sync(orig->unit[i]);
        } else if (orig->unit[i]->isAlive()) {
            saveUnit(new Unit(*(orig->unit[i])));
        }
    }
}

void UnitContainer::sync(UnitContainer* orig, PLAYER plyr) {
    if (latestuuid < orig->latestuuid) {
        latestuuid = orig->latestuuid;
    } else {
        orig->latestuuid = latestuuid;
    }
    for( unsigned int i=0; i<orig->unit.size(); i++){
        if(orig->unit[i]->getOwner() == plyr
           || plyr == PLAYER0
          ){
            // load unit with same uuid
            Unit* tmp = getUnit(orig->unit[i]->getUuid());
            if (tmp != NULL){
                // sync it, if it is already known
                tmp->sync(orig->unit[i]);
            } else if (orig->unit[i]->isAlive()) {
                saveUnit(new Unit(*(orig->unit[i])));
            }
        }
    }
}

std::string UnitContainer::getAscii() {
    std::stringstream out;
    for (unsigned int i=0; i < unit.size(); i++) {
        out << unit[i]->getAsciiInfos() << std::endl;
    }
    return out.str();
}

void UnitContainer::giveRandomOrders(PLAYER p) {
    srand( (unsigned int) time(NULL));
    for (unsigned int i=0; i < unit.size(); i++) {
        Unit* u = unit[i];
        if (u->getOwner() == p) {
            u->clearCommands();
            bool cmd = false;
            unsigned char tries = 0;
            Coordinates pos;
            while (!cmd  && (tries < 27)) {
                pos = u->getPos();
                pos.x += rand()%7 - 3;
                pos.y += rand()%7 - 3;
                if (u->command(pos)) {
                    if (goodCommand(i)) {
                        cmd = true;
                    } else {
                        u->clearCommands();
                    }
                }
                tries ++;
            }
        }
    }
}

Unit* UnitContainer::getUnit(Coordinates pos) {
    for(unsigned int i = 0; i<unit.size(); i++){
        if( unit[i]->getPos() == pos &&
            unit[i]->isAlive()
          ){
            return (unit[i]);
        }
    }
    return NULL;
}

Unit* UnitContainer::getSelectedUnit() {
    if (selectedunit >= 0){
        return (unit[selectedunit]);
    } else {
        return NULL;
    }
}

void UnitContainer::saveUnit(Unit* u) {
    if (u != NULL){
        if( u->getUuid() == 0){
            latestuuid++;
            u->setUuid(latestuuid);
        } else {
            for( unsigned int i = 0; i<unit.size(); i++){
                if( (unit[i])->getUuid() == u->getUuid()){
                    return;
                }
            }
        }
        if (u->getUuid() > latestuuid){
            latestuuid = u->getUuid();
        }
        unit.push_back(u);
        u->setMap(map);
    }
}

void UnitContainer::delUnit(unsigned int uuid) {
    for (unsigned int i=0; i<unit.size(); i++) {
        if (unit[i]->getUuid() == uuid) {
            unit.erase(unit.begin()+i);
            return;
        }
    }
    if (selectedunit >= (int) unit.size()) {
        selectedunit = -1;
    }
}

bool UnitContainer::checkForUnit(UNITTYPE type, PLAYER p) {
    for (unsigned int i=0; i<unit.size(); i++){
        if( unitMatches(i, type, p) && unit[i]->isAlive() ){
            return true;
        }
    }
    return false;
}

Unit* UnitContainer::selectNextUnit(UNITTYPE type, PLAYER p) {
    if (unit.size() > 0) {
        unsigned int i;
        if ( (unsigned int) (selectedunit+1) < unit.size()){
            i = (unsigned int) selectedunit+1;
        } else {
            i = 0;
        }
        while(selectedunit != (int) i){
            if( unitMatches(i, type, p) ) {
                selectedunit = i;
                return (unit[i]);
            }
            if ( i+1 < unit.size()){
                i++;
            } else {
                if (selectedunit < 0) {
                    return NULL;
                } else {
                    i = 0;
                }
            }
        }
        if( unitMatches(i, type, p) ) {
            selectedunit = i;
            return (unit[i]);
        }
    }
    return NULL;
}

Unit* UnitContainer::selectPrevUnit(UNITTYPE type, PLAYER p) {
    if (unit.size() > 0) {
        unsigned int i;
        if ( selectedunit <= 0){
            i = unit.size() -1;
        } else {
            i = (unsigned int) (selectedunit-1);
        }
        while(selectedunit != (int) i){
            if( unitMatches(i, type, p) ) {
                selectedunit = i;
                return (unit[i]);
            }
            if ( i > 0){
                i--;
            } else {
                if (selectedunit < 0) {
                    return NULL;
                } else {
                    i = unit.size() -1;
                }
            }
        }
        if( unitMatches(i, type, p) ) {
            selectedunit = i;
            return (unit[i]);
        }
    }
    return NULL;
}

Unit* UnitContainer::selectUnit(Coordinates pos) {
    for(unsigned int i = 0; i<unit.size(); i++){
        if( (unit[i])->getPos() == pos && unit[i]->isAlive()){
            selectedunit = i;
            return (unit[i]);
        }
    }
    selectedunit = -1;
    return NULL;
}

bool UnitContainer::unitMatches(unsigned int i, UNITTYPE t, PLAYER p) {
    return  (( t == NOUNIT  || t == unit[i]->getType()  ) &&
             ( p == PLAYER0 || p == unit[i]->getOwner() ) );
}

bool UnitContainer::collide(Unit* u1, Unit* u2) {
    // test whether the two units collide
    switch (u1->getPos().distanceTo(u2->getPos())) {
    case 0:
        break;
    case 1:
        if((u1->getPos() == u2->getOldWay().front()) &&
           (u2->getPos() == u1->getOldWay().front()) ) {
            break;
        } else {
            return false;
        }
    default:
        return false;
    }
    if((u1->getOwner() == u2->getOwner()) ||
       !u1->isAlive()  || !u2->isAlive()  ||
        u1->getType() == SUMMONER ||
        u2->getType() == SUMMONER ) {
        return false;
    }
    // if yes, let them fight
    if (u1->getType() == SPIRIT) {
        u2->changeHp(-5);
        u1->kill();
        return true;
    }
    if (u2->getType() == SPIRIT) {
        u1->changeHp(-5);
        u2->kill();
        return true;
    }
    if( u1->getType() == TERRAFORMER &&
        u2->getType() == WARRIOR) {
        map->setHeight(u2->getPos(), map->getHeight(u2->getPos())-1);
        u1->kill();
        u2->kill();
        return true;
    }
    if( u1->getType() == WARRIOR &&
        u2->getType() == TERRAFORMER) {
        map->setHeight(u1->getPos(), map->getHeight(u1->getPos())-1);
        u1->kill();
        u2->kill();
        return true;
    }

    srand( (unsigned int) time(NULL));
    float pu1victim = 0.25f
                    + 0.5f / (1.0f +(((float)(u1->getSpeed())
                                  /((float)(u2->getSpeed()))))
                             );
    while (u1->isAlive() && u2->isAlive()) {
        if ((float)(rand()) / (float)(RAND_MAX) < pu1victim ) {
            u1->changeHp(damage(u2->getHp(),u1->getMaxHp()));
        } else {
            u2->changeHp(damage(u1->getHp(),u2->getMaxHp()));
        }
    }
    return true;
}

short UnitContainer::damage(short att, short def) {
    return (-1 - (short) (9.0f * exp(- (float) def / (float) att)));
}

bool UnitContainer::goodCommand(unsigned int u) {
    std::vector<Coordinates> way = unit[u]->getWay();
    Coordinates c;

    /* get the target field
     *
     * note: vector counts zero based
     * and current position is not in way
     */
    if (unit[u]->getType() != SUMMONER) {
        if (way.size() >= unit[u]->getSpeed()) {
            c = way[unit[u]->getSpeed() -1];
        } else {
            c = way.back();
        }
    } else {
        c = unit[u]->getPos();
    }


    if (map->getTerrain(c) == WATER) {
        return false;
    }
    for (unsigned int i=0; i<unit.size(); i++) {
        if ((i!=u) && (unit[i]->getOwner() == unit[u]->getOwner())) {
            std::vector<Coordinates> otherway = unit[i]->getWay();
            if (otherway.size() >= unit[i]->getSpeed()) {
                if (otherway[unit[i]->getSpeed()-1] == c){
                    return false;
                }
            } else {
                if (otherway.empty()) {
                    if (unit[i]->getPos() == c) {
                        return false;
                    }
                } else {
                    if (otherway.back() == c) {
                        return false;
                    }
                }
            }
        }
    }

    Coordinates pos = unit[u]->getPos();
    unsigned int oldvisdex = map->getVisDex();
    unit[u]->setPos(c);
    unsigned int newvisdex = map->getVisDex();
    unit[u]->setPos(pos);

    if (oldvisdex > newvisdex) {
        return false;
    } else {
        return true;
    }
}
